'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isUserSignedIn = isUserSignedIn;
exports.redirectUserToSignIn = redirectUserToSignIn;
exports.getAuthResponseToken = getAuthResponseToken;
exports.isSignInPending = isSignInPending;
exports.signUserIn = signUserIn;
exports.loadUserData = loadUserData;
exports.signUserOut = signUserOut;
exports.getAuthRequestFromURL = getAuthRequestFromURL;
exports.fetchAppManifest = fetchAppManifest;
exports.redirectUserToApp = redirectUserToApp;

var _queryString = require('query-string');

var _queryString2 = _interopRequireDefault(_queryString);

var _jsontokens = require('jsontokens');

var _authMessages = require('./authMessages');

var _index = require('../index');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var BLOCKSTACK_HANDLER = "blockstack";
var BLOCKSTACK_STORAGE_LABEL = "blockstack";
var DEFAULT_BLOCKSTACK_HOST = "https://blockstack.org/auth";

/**
 * For applications
 */

function isUserSignedIn() {
  return window.localStorage.getItem(BLOCKSTACK_STORAGE_LABEL) ? true : false;
}

function redirectUserToSignIn(authRequest) {
  var blockstackIDHost = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : DEFAULT_BLOCKSTACK_HOST;

  setTimeout(function () {
    window.location = blockstackIDHost + "?authRequest=" + authRequest;
  }, 500);

  window.location = BLOCKSTACK_HANDLER + ":" + authRequest;
}

function getAuthResponseToken() {
  var queryDict = _queryString2.default.parse(location.search);
  return queryDict.authResponse ? queryDict.authResponse : null;
}

function isSignInPending() {
  return getAuthResponseToken() ? true : false;
}

function signUserIn(callbackFunction) {
  var authResponseToken = getAuthResponseToken();

  if ((0, _authMessages.verifyAuthResponse)(authResponseToken)) {
    var tokenPayload = (0, _jsontokens.decodeToken)(authResponseToken).payload;
    var userData = {
      username: tokenPayload.username,
      profile: tokenPayload.profile,
      authResponseToken: authResponseToken
    };
    window.localStorage.setItem(BLOCKSTACK_STORAGE_LABEL, JSON.stringify(userData));
    callbackFunction(true);
  } else {
    callbackFunction(false);
  }
}

function loadUserData(callbackFunction) {
  var userData = JSON.parse(window.localStorage.getItem(BLOCKSTACK_STORAGE_LABEL));
  callbackFunction(userData);
}

function signUserOut(redirectURL) {
  window.localStorage.removeItem(BLOCKSTACK_STORAGE_LABEL);
  window.location = redirectURL;
}

/**
 * For identity providers
 */

function getAuthRequestFromURL() {
  var queryDict = _queryString2.default.parse(location.search);
  if (queryDict.authRequest !== null && queryDict.authRequest !== undefined) {
    return queryDict.authRequest.split(BLOCKSTACK_HANDLER + ':').join('');
  } else {
    return null;
  }
}

function fetchAppManifest(authRequest) {
  return new Promise(function (resolve, reject) {
    if (!authRequest) {
      reject("Invalid auth request");
    } else {
      var payload = (0, _jsontokens.decodeToken)(authRequest).payload;
      var manifestURI = payload.manifest_uri;
      try {
        fetch(manifestURI).then(function (response) {
          return response.text();
        }).then(function (responseText) {
          return JSON.parse(responseText);
        }).then(function (responseJSON) {
          resolve(responseJSON);
        }).catch(function (e) {
          console.log(e.stack);
          reject("URI request couldn't be completed");
        });
      } catch (e) {
        console.log(e.stack);
        reject("URI request couldn't be completed");
      }
    }
  });
}

function redirectUserToApp(authRequest, authResponse) {
  var payload = (0, _jsontokens.decodeToken)(authRequest).payload;
  var redirectURI = payload.redirect_uri;
  if (redirectURI) {
    redirectURI = (0, _index.updateQueryStringParameter)(redirectURI, 'authResponse', authResponse);
  } else {
    throw new Error("Invalid redirect URI");
  }
  window.location = redirectURI;
}